@@ -27,6 +27,7 @@
 
 #include <gtk/gtk.h>
 #include <gio/gio.h>
+#include <pango/pango.h>
 #include <libnemo-private/nemo-file-attributes.h>
 #include <libnemo-private/nemo-icon-info.h>
 
@@ -105,16 +106,19 @@ typedef enum {
     NEMO_FILE_TOOLTIP_FLAGS_CREATED_DATE = (1<<4)
 } NemoFileTooltipFlags;
 
+typedef enum {
+    NEMO_FILE_LOAD_DEFERRED_ATTRS_NO,
+    NEMO_FILE_LOAD_DEFERRED_ATTRS_YES,
+    NEMO_FILE_LOAD_DEFERRED_ATTRS_PRELOAD
+} NemoFileLoadDeferredAttrs;
+
 /* Emblems sometimes displayed for NemoFiles. Do not localize. */ 
 #define NEMO_FILE_EMBLEM_NAME_SYMBOLIC_LINK "symbolic-link"
 #define NEMO_FILE_EMBLEM_NAME_CANT_READ "noread"
 #define NEMO_FILE_EMBLEM_NAME_CANT_WRITE "nowrite"
 #define NEMO_FILE_EMBLEM_NAME_TRASH "trash"
 #define NEMO_FILE_EMBLEM_NAME_NOTE "note"
-
-/* Used in list view and tree-sidebar to mark pinned files */
-#define UNPINNED_TEXT_WEIGHT 400
-#define PINNED_TEXT_WEIGHT 1000
+#define NEMO_FILE_EMBLEM_NAME_FAVORITE "xapp-favorite"
 
 typedef void (*NemoFileCallback)          (NemoFile  *file,
 				               gpointer       callback_data);
@@ -130,8 +134,8 @@ typedef char * (*NemoTruncateCallback)    (const char    *string,
 					       int	      width,
 					       void	     *context);
 
-
 #define NEMO_FILE_ATTRIBUTES_FOR_ICON (NEMO_FILE_ATTRIBUTE_INFO | NEMO_FILE_ATTRIBUTE_LINK_INFO | NEMO_FILE_ATTRIBUTE_THUMBNAIL)
+#define NEMO_FILE_DEFERRED_ATTRIBUTES (NEMO_FILE_ATTRIBUTE_THUMBNAIL | NEMO_FILE_ATTRIBUTE_EXTENSION_INFO)
 
 typedef void NemoFileListHandle;
 
@@ -222,6 +226,8 @@ gboolean                nemo_file_is_user_special_directory         (NemoFile
 gboolean		nemo_file_is_archive			(NemoFile			*file);
 gboolean                nemo_file_is_in_trash                       (NemoFile                   *file);
 gboolean                nemo_file_is_in_recent                      (NemoFile                   *file);
+gboolean                nemo_file_is_in_favorites                   (NemoFile                   *file);
+gboolean                nemo_file_is_unavailable_favorite           (NemoFile                   *file);
 gboolean                nemo_file_is_in_admin                       (NemoFile                   *file);
 gboolean                nemo_file_is_in_desktop                     (NemoFile                   *file);
 gboolean		nemo_file_is_home				(NemoFile                   *file);
@@ -240,6 +246,7 @@ NemoRequestStatus   nemo_file_get_deep_counts                   (NemoFile
 									 gboolean                        force);
 gboolean                nemo_file_should_show_thumbnail             (NemoFile                   *file);
 void                    nemo_file_delete_thumbnail                  (NemoFile                   *file);
+gboolean                nemo_file_has_loaded_thumbnail              (NemoFile                   *file);
 gboolean                nemo_file_should_show_directory_item_count  (NemoFile                   *file);
 gboolean                nemo_file_should_show_type                  (NemoFile                   *file);
 GList *                 nemo_file_get_keywords                      (NemoFile                   *file);
@@ -521,7 +528,11 @@ void     nemo_file_set_is_desktop_orphan          (NemoFile *file, gboolean is_d
 
 gboolean nemo_file_get_pinning                    (NemoFile *file);
 void     nemo_file_set_pinning                    (NemoFile *file, gboolean  pin);
-
+gboolean nemo_file_get_is_favorite                (NemoFile *file);
+void     nemo_file_set_is_favorite                (NemoFile *file, gboolean favorite);
+void     nemo_file_set_load_deferred_attrs        (NemoFile *file,
+                                                   NemoFileLoadDeferredAttrs load_deferred_attrs);
+NemoFileLoadDeferredAttrs nemo_file_get_load_deferred_attrs (NemoFile *file);
 /* Debugging */
 void                    nemo_file_dump                              (NemoFile                   *file);
 
