@@ -48,7 +48,6 @@
 #include <string.h>
 
 /* gap between bottom of icon and start of text box */
-#define LABEL_OFFSET 3
 #define LABEL_OFFSET_BESIDES 3
 #define LABEL_LINE_SPACING 0
 
@@ -118,6 +117,7 @@ struct NemoIconCanvasItemDetails {
 	guint is_visible : 1;
 
     guint is_pinned : 1;
+    guint fav_unavailable : 1;
 
 	/* Cached PangoLayouts. Only used if the icon is visible */
 	PangoLayout *editable_text_layout;
@@ -146,7 +146,8 @@ enum {
     	PROP_HIGHLIGHTED_AS_KEYBOARD_FOCUS,
     	PROP_HIGHLIGHTED_FOR_DROP,
 	PROP_HIGHLIGHTED_FOR_CLIPBOARD,
-    PROP_PINNED
+    PROP_PINNED,
+    PROP_FAV_UNAVAILABLE
 };
 
 typedef enum {
@@ -370,6 +371,14 @@ nemo_icon_canvas_item_set_property (GObject        *object,
         nemo_icon_canvas_item_invalidate_label (item);
 
         break;
+    case PROP_FAV_UNAVAILABLE:
+        if (!details->fav_unavailable == !g_value_get_boolean (value)) {
+            return;
+        }
+        details->fav_unavailable = g_value_get_boolean (value);
+        nemo_icon_canvas_item_invalidate_label (item);
+
+        break;
 
 	default:
 		g_warning ("nemo_icons_view_item_item_set_arg on unknown argument");
@@ -584,7 +593,6 @@ compute_text_rectangle (const NemoIconCanvasItem *item,
 {
 	EelIRect text_rectangle;
 	double pixels_per_unit;
-    double label_offset;
 	double text_width, text_height, text_height_for_layout, text_height_for_entire_text, real_text_height, text_dx;
 
 	pixels_per_unit = EEL_CANVAS_ITEM (item)->canvas->pixels_per_unit;
@@ -594,14 +602,12 @@ compute_text_rectangle (const NemoIconCanvasItem *item,
 		text_height_for_layout = item->details->text_height_for_layout;
 		text_height_for_entire_text = item->details->text_height_for_entire_text;
 		text_dx = item->details->text_dx;
-        label_offset = LABEL_OFFSET;
 	} else {
 		text_width = item->details->text_width / pixels_per_unit;
 		text_height = item->details->text_height / pixels_per_unit;
 		text_height_for_layout = item->details->text_height_for_layout / pixels_per_unit;
 		text_height_for_entire_text = item->details->text_height_for_entire_text / pixels_per_unit;
 		text_dx = item->details->text_dx / pixels_per_unit;
-        label_offset = LABEL_OFFSET / pixels_per_unit;
 	}
 
 	if (NEMO_ICON_CONTAINER (EEL_CANVAS_ITEM (item)->canvas)->details->label_position == NEMO_ICON_LABEL_POSITION_BESIDE) {
@@ -644,7 +650,7 @@ compute_text_rectangle (const NemoIconCanvasItem *item,
         text_rectangle.y1 = text_rectangle.y0 + real_text_height;
 	} else {
         text_rectangle.x0 = (icon_rectangle.x0 + icon_rectangle.x1) / 2 - (int) text_width / 2;
-        text_rectangle.y0 = icon_rectangle.y1 + label_offset;
+        text_rectangle.y0 = icon_rectangle.y1;
         text_rectangle.x1 = text_rectangle.x0 + text_width;
 
 		if (usage == BOUNDS_USAGE_FOR_LAYOUT) {
@@ -652,12 +658,12 @@ compute_text_rectangle (const NemoIconCanvasItem *item,
 		} else if (usage == BOUNDS_USAGE_FOR_ENTIRE_ITEM) {
 			real_text_height = text_height_for_entire_text;
 		} else if (usage == BOUNDS_USAGE_FOR_DISPLAY) {
-			real_text_height = text_height;
+			real_text_height = text_height + 6; /* Extra bottom highlight padding. */
 		} else {
 			g_assert_not_reached ();
 		}
 
-		text_rectangle.y1 = text_rectangle.y0 + real_text_height + label_offset;
+		text_rectangle.y1 = text_rectangle.y0 + real_text_height;
         }
 
 	return text_rectangle;
@@ -809,6 +815,7 @@ layout_get_size_for_layout (PangoLayout *layout,
 
 #define TEXT_BACK_PADDING_X 4
 #define TEXT_BACK_PADDING_Y 1
+#define TEXT_TOP_GAP 3
 
 static void
 prepare_pango_layout_width (NemoIconCanvasItem *item,
@@ -845,20 +852,15 @@ prepare_pango_layout_for_draw (NemoIconCanvasItem *item,
 {
 	NemoIconCanvasItemDetails *details;
 	NemoIconContainer *container;
-	gboolean needs_highlight;
 
 	prepare_pango_layout_width (item, layout);
 
 	container = NEMO_ICON_CONTAINER (EEL_CANVAS_ITEM (item)->canvas);
 	details = item->details;
 
-	needs_highlight = details->is_highlighted_for_selection || details->is_highlighted_for_drop;
-
 	if (IS_COMPACT_VIEW (container)) {
 		pango_layout_set_height (layout, -1);
-	} else if (needs_highlight ||
-		   details->is_prelit ||
-		   details->is_highlighted_as_keyboard_focus ||
+	} else if (details->is_prelit ||
 		   details->entire_text) {
 		/* VOODOO-TODO, cf. compute_text_rectangle() */
 		pango_layout_set_height (layout, G_MININT);
@@ -1117,7 +1119,7 @@ draw_label_text (NemoIconCanvasItem *item,
 		gtk_style_context_set_state (context, state);
 
 		gtk_render_layout (context, cr,
-				   x, text_rect.y0 + TEXT_BACK_PADDING_Y,
+				   x, text_rect.y0 + TEXT_TOP_GAP,
 				   editable_layout);
 
 		gtk_style_context_restore (context);
@@ -1139,7 +1141,7 @@ draw_label_text (NemoIconCanvasItem *item,
 		gtk_style_context_add_class (context, "dim-label");
 
 		gtk_render_layout (context, cr,
-				   x, text_rect.y0 + details->editable_text_height + LABEL_LINE_SPACING + TEXT_BACK_PADDING_Y,
+				   x, text_rect.y0 + details->editable_text_height + LABEL_LINE_SPACING + TEXT_TOP_GAP,
 				   additional_layout);
 	}
 
@@ -1474,8 +1476,15 @@ create_label_layout (NemoIconCanvasItem *item,
                                          container->details->font_size_table [container->details->zoom_level]);
     }
 
+    if (item->details->fav_unavailable) {
+        pango_font_description_set_weight (desc, UNAVAILABLE_TEXT_WEIGHT);
+    }
+    else
     if (item->details->is_pinned) {
-        pango_font_description_set_weight (desc, PANGO_WEIGHT_BOLD);
+        pango_font_description_set_weight (desc, PINNED_TEXT_WEIGHT);
+    }
+    else {
+        pango_font_description_set_weight (desc, NORMAL_TEXT_WEIGHT);
     }
 
 	pango_layout_set_font_description (layout, desc);
@@ -2022,6 +2031,32 @@ nemo_icon_canvas_item_set_entire_text (NemoIconCanvasItem       *item,
 	}
 }
 
+gint
+nemo_icon_canvas_item_get_fixed_text_height_for_layout (NemoIconCanvasItem *item)
+{
+    NemoIconContainer *container;
+    PangoLayout *layout;
+    gint line_height, total_height, lines;
+
+    container = NEMO_ICON_CONTAINER (EEL_CANVAS_ITEM (item)->canvas);
+
+    if (nemo_icon_container_get_zoom_level (container) == NEMO_ZOOM_LEVEL_SMALLEST) {
+        // No label/info, just a bit of padding.
+        return 0;
+    }
+
+    lines = nemo_icon_container_get_max_layout_lines (container);
+    lines += nemo_icon_container_get_additional_text_line_count (container);
+
+    layout = create_label_layout (item, "-");
+    pango_layout_get_pixel_size (layout, NULL, &line_height);
+
+    total_height = (line_height * lines) + (LABEL_LINE_SPACING * (lines - 1));
+    g_object_unref (layout);
+
+    return total_height;
+}
+
 /* Class initialization function for the icon canvas item. */
 static void
 nemo_icon_canvas_item_class_init (NemoIconCanvasItemClass *class)
@@ -2093,6 +2128,14 @@ nemo_icon_canvas_item_class_init (NemoIconCanvasItemClass *class)
                               "backing file is pinned",
                       FALSE, G_PARAM_READWRITE));
 
+    g_object_class_install_property (
+        object_class,
+        PROP_FAV_UNAVAILABLE,
+        g_param_spec_boolean ("fav-unavailable",
+                              "fav-unavailable",
+                              "backing file is a favorite and is not reachable",
+                      FALSE, G_PARAM_READWRITE));
+
 	item_class->update = nemo_icon_canvas_item_update;
 	item_class->draw = nemo_icon_canvas_item_draw;
 	item_class->point = nemo_icon_canvas_item_point;
