@@ -70,10 +70,11 @@ static const GtkTargetEntry drag_types [] = {
 
 static const GtkTargetEntry drop_types [] = {
 	{ (char *)NEMO_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NEMO_ICON_DND_GNOME_ICON_LIST },
+	/* prefer XDS Protocol Type over "_NETSCAPE_URL" to make DnD work in Firefox. */
+	{ (char *)NEMO_ICON_DND_XDNDDIRECTSAVE_TYPE, 0, NEMO_ICON_DND_XDNDDIRECTSAVE },
 	/* prefer "_NETSCAPE_URL" over "text/uri-list" to satisfy web browsers. */
 	{ (char *)NEMO_ICON_DND_NETSCAPE_URL_TYPE, 0, NEMO_ICON_DND_NETSCAPE_URL },
 	{ (char *)NEMO_ICON_DND_URI_LIST_TYPE, 0, NEMO_ICON_DND_URI_LIST },
-	{ (char *)NEMO_ICON_DND_XDNDDIRECTSAVE_TYPE, 0, NEMO_ICON_DND_XDNDDIRECTSAVE }, /* XDS Protocol Type */
 	{ (char *)NEMO_ICON_DND_RAW_TYPE, 0, NEMO_ICON_DND_RAW },
 	/* Must be last: */
 	{ (char *)NEMO_ICON_DND_ROOTWINDOW_DROP_TYPE,  0, NEMO_ICON_DND_ROOTWINDOW_DROP }
@@ -94,6 +95,10 @@ initialize_dnd_grid (NemoIconContainer *container)
 {
     GList *selection, *p;
 
+    if (!NEMO_ICON_CONTAINER_GET_CLASS (container)->is_grid_container) {
+        return;
+    }
+
     if (container->details->dnd_grid != NULL) {
         nemo_centered_placement_grid_free (container->details->dnd_grid);
     }
@@ -1029,13 +1034,24 @@ handle_nonlocal_move (NemoIconContainer *container,
 		x = nemo_icon_container_get_canvas_width (container, allocation) - x;
 	}
 
-	/* start the copy */
-	g_signal_emit_by_name (container, "move_copy_items",
-			       source_uris,
-			       source_item_locations,
-			       target_uri,
-			       action,
-			       x, y);
+    if (g_strcmp0 (target_uri, "favorites:///") == 0) {
+        GList *l;
+
+        for (l = source_uris; l != NULL; l = l->next) {
+            gchar *uri = (gchar *) l->data;
+            NemoFile *source_file = nemo_file_get_by_uri (uri);
+            nemo_file_set_is_favorite (source_file, TRUE);
+            nemo_file_unref (source_file);
+        }
+    } else {
+        /* start the copy */
+        g_signal_emit_by_name (container, "move_copy_items",
+                               source_uris,
+                               source_item_locations,
+                               target_uri,
+                               action,
+                               x, y);
+    }
 
 	if (free_target_uri) {
 		g_free ((char *)target_uri);
@@ -1443,6 +1459,7 @@ nemo_icon_container_get_drop_action (NemoIconContainer *container,
 {
 	char *drop_target;
 	gboolean icon_hit;
+    gboolean favorites_target;
 	double world_x, world_y;
 
 	icon_hit = FALSE;
@@ -1455,6 +1472,22 @@ nemo_icon_container_get_drop_action (NemoIconContainer *container,
 	canvas_widget_to_world (EEL_CANVAS (container), x, y, &world_x, &world_y);
 	*action = 0;
 
+    drop_target = nemo_icon_container_find_drop_target (container,
+                                context, x, y, &icon_hit, FALSE);
+
+    favorites_target = FALSE;
+
+    if (g_strcmp0 (drop_target, "favorites:///") == 0) {
+        favorites_target = TRUE;
+    }
+
+    g_free (drop_target);
+
+    if (favorites_target) {
+        *action = GDK_ACTION_COPY;
+        return;
+    }
+
 	/* case out on the type of object being dragged */
 	switch (container->details->dnd_info->drag_info.data_type) {
 	case NEMO_ICON_DND_GNOME_ICON_LIST:
