@@ -395,6 +395,7 @@ get_drop_action (NemoTreeViewDragDest *dest,
 {
 	char *drop_target;
 	int action;
+    gboolean favorites_target = FALSE;;
 
 	if (!dest->details->have_drag_data ||
 	    (dest->details->drag_type == NEMO_ICON_DND_GNOME_ICON_LIST &&
@@ -402,6 +403,18 @@ get_drop_action (NemoTreeViewDragDest *dest,
 		return 0;
 	}
 
+    drop_target = get_drop_target_uri_for_path (dest, path);
+
+    if (g_strcmp0 (drop_target, "favorites:///") == 0) {
+        favorites_target = TRUE;
+    }
+
+    g_free (drop_target);
+
+    if (favorites_target) {
+        return GDK_ACTION_COPY;
+    }
+
 	switch (dest->details->drag_type) {
 	case NEMO_ICON_DND_GNOME_ICON_LIST :
 		drop_target = get_drop_target_uri_for_path (dest, path);
@@ -617,14 +630,25 @@ receive_uris (NemoTreeViewDragDest *dest,
 	}
 
 	if (real_action > 0) {
-		if (!nemo_drag_uris_local (drop_target, source_uris)
-			|| real_action != GDK_ACTION_MOVE) {
-			g_signal_emit (dest, signals[MOVE_COPY_ITEMS], 0,
-				       source_uris,
-				       drop_target,
-				       real_action,
-				       x, y);
-		}
+        if (g_strcmp0 (drop_target, "favorites:///") == 0) {
+            GList *l;
+
+            for (l = source_uris; l != NULL; l = l->next) {
+                const gchar *uri = l->data;
+                NemoFile *source_file = nemo_file_get_by_uri (uri);
+                nemo_file_set_is_favorite (source_file, TRUE);
+                nemo_file_unref (source_file);
+            }
+        } else {
+            if (!nemo_drag_uris_local (drop_target, source_uris)
+                || real_action != GDK_ACTION_MOVE) {
+                g_signal_emit (dest, signals[MOVE_COPY_ITEMS], 0,
+                           source_uris,
+                           drop_target,
+                           real_action,
+                           x, y);
+            }            
+        }
 	}
 
 	g_free (drop_target);
