@@ -37,6 +37,7 @@
 #include <glib/gstdio.h>
 #include <string.h>
 #include <gdk/gdkx.h>
+#include <libxapp/xapp-favorites.h>
 
 #include <libnemo-private/nemo-file-attributes.h>
 #include <libnemo-private/nemo-file.h>
@@ -1114,13 +1115,13 @@ open_with_dialog_response_cb (GtkDialog *dialog,
     GAppInfo *info;
     GList files;
 
-    parent_window = user_data;
-
     if (response_id != GTK_RESPONSE_OK) {
         gtk_widget_destroy (GTK_WIDGET (dialog));
         return;
     }
 
+    parent_window = user_data;
+
     GtkWidget *content = gtk_dialog_get_content_area (dialog);
     GList *children = gtk_container_get_children (GTK_CONTAINER (content));
 
@@ -1211,8 +1212,8 @@ application_unhandled_uri (ActivateParameters *parameters, char *uri)
 {
 
     NemoFile *file = nemo_file_get_existing_by_uri (uri);
-
-    char *primary, *secondary, *display_name;
+    gboolean enable_exec_button;
+    char *primary, *secondary, *display_name, *param_uri;
     GtkWidget *dialog;
 
     ActivateParametersSpecial *parameters_special;
@@ -1224,37 +1225,55 @@ application_unhandled_uri (ActivateParameters *parameters, char *uri)
     }
     parameters_special->file = nemo_file_ref (file);
 
-    primary = _("Unknown file type");
-    display_name = nemo_file_get_display_name (file);
-    secondary =
-        g_strdup_printf (_("The file \"%s\" has no known programs associated with it.  "
-                   "If you trust the source of this file, and have sufficient permissions, you can mark it executable and launch it.  "
-                   "Or, you can use the Open With dialog to pick a program to associate it with."
-                   ),
-                 display_name);
+    enable_exec_button = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_MIME_MAKE_EXECUTABLE);
 
     dialog = gtk_message_dialog_new (parameters->parent_window,
                      0,
                      GTK_MESSAGE_WARNING,
                      GTK_BUTTONS_NONE,
                      NULL);
+
+    primary = _("Unknown file type");
+    display_name = nemo_file_get_display_name (file);
+    if (display_name == NULL || g_strcmp0(display_name, "") == 0) {
+        g_free(display_name);
+        display_name = g_strdup(uri);
+    }
+    if (enable_exec_button) {
+        secondary =
+            g_strdup_printf (_("The file \"%s\" has no known programs associated with it.  "
+                             "If you trust the source of this file, and have sufficient permissions, you can mark it executable and launch it.  "
+                             "Or, you can use the Open With dialog to pick a program to associate it with."
+                             ),
+                             display_name);
+
+        gtk_dialog_add_button (GTK_DIALOG (dialog),
+                               _("Make executable and run"), RESPONSE_RUN);
+    } else {
+        secondary = g_strdup_printf (_("The file \"%s\" has no known programs associated with it.  "
+                                     "Use the Open With dialog to pick a program to open it with."),
+                                     display_name);
+    }
+
     g_object_set (dialog,
               "text", primary,
               "secondary-text", secondary,
               NULL);
 
-    gtk_dialog_add_button (GTK_DIALOG (dialog),
-                   _("Make executable and run"), RESPONSE_RUN);
-    gtk_dialog_add_button (GTK_DIALOG (dialog),
+    param_uri = nemo_file_get_uri (parameters_special->file);
+    if (param_uri != NULL) {
+        gtk_dialog_add_button (GTK_DIALOG (dialog),
                    _("Choose a program"), RESPONSE_OPEN_WITH);
+    }
 
-    if (!nemo_file_can_set_permissions (file)) {
+    if (!nemo_file_can_set_permissions (file) && enable_exec_button) {
         GtkWidget *w = gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), RESPONSE_RUN);
         gtk_widget_set_sensitive (w, FALSE);
     }
 
     gtk_dialog_add_button (GTK_DIALOG (dialog),
-                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
     g_signal_connect (dialog, "response",
@@ -1264,6 +1283,7 @@ application_unhandled_uri (ActivateParameters *parameters, char *uri)
 
     g_free (display_name);
     g_free (secondary);
+    g_free (param_uri);
     return;
 }
 
@@ -1631,6 +1651,13 @@ activate_files (ActivateParameters *parameters)
 		for (l = unhandled_open_in_app_uris; l != NULL; l = l->next) {
 			uri = l->data;
 
+            // TODO: Handle this better? - this unhandled uri is a favorite with a missing target.
+            XAppFavoriteInfo *info = xapp_favorites_find_by_uri (xapp_favorites_get_default (), uri);
+
+            if (info != NULL) {
+                continue;
+            }
+
 			/* this does not block */
 			application_unhandled_uri (parameters, uri);
 		}
