@@ -97,6 +97,8 @@ struct FMTreeViewDetails {
 	GtkWidget *popup_copy;
 	GtkWidget *popup_paste;
 	GtkWidget *popup_rename;
+    GtkWidget *popup_pin;
+    GtkWidget *popup_unpin;
 	GtkWidget *popup_trash;
 	GtkWidget *popup_delete;
 	GtkWidget *popup_properties;
@@ -110,8 +112,10 @@ struct FMTreeViewDetails {
 	guint selection_changed_timer;
 
     NemoActionManager *action_manager;
+    guint action_manager_changed_id;
     GList *action_items;
     guint hidden_files_changed_id;
+    guint sort_directories_first : 1;
 };
 
 typedef struct {
@@ -453,14 +457,13 @@ selection_changed_timer_callback(FMTreeView *view)
 	NemoFileAttributes attributes;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
-	
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->details->tree_widget));
 
 	/* no activation if popup menu is open */
 	if (view->details->popup_file != NULL) {
 		return FALSE;
 	}
 
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->details->tree_widget));
 	cancel_activation (view);
 
 	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
@@ -537,9 +540,10 @@ compare_rows (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer call
 	} else if (file_b == NULL) {
 		result = +1;
 	} else {
-		result = nemo_file_compare_for_sort (file_a, file_b,
-							 NEMO_FILE_SORT_BY_DISPLAY_NAME,
-							 FALSE, FALSE);
+        result = nemo_file_compare_for_sort (file_a, file_b,
+                                             NEMO_FILE_SORT_BY_DISPLAY_NAME,
+                                             FM_TREE_VIEW (callback_data)->details->sort_directories_first,
+                                             FALSE);
 	}
 
 	nemo_file_unref (file_a);
@@ -781,6 +785,14 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 			gtk_widget_hide (view->details->popup_unmount_separator);
 		}
 
+        if (nemo_file_get_pinning (view->details->popup_file)) {
+            gtk_widget_hide (view->details->popup_pin);
+            gtk_widget_show (view->details->popup_unpin);
+        } else {
+            gtk_widget_show (view->details->popup_pin);
+            gtk_widget_hide (view->details->popup_unpin);
+        }
+
         gboolean actions_visible = FALSE;
 
         GList *l;
@@ -792,8 +804,13 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 
         for (l = view->details->action_items; l != NULL; l = l->next) {
             p = l->data;
-            if (nemo_action_get_visibility (p->action, tmp, parent)) {
-                gtk_menu_item_set_label (GTK_MENU_ITEM (p->item), nemo_action_get_label (p->action, tmp, parent));
+            if (nemo_action_get_visibility (p->action, tmp, parent, FALSE)) {
+                gchar *action_label;
+
+                action_label = nemo_action_get_label (p->action, tmp, parent);
+                gtk_menu_item_set_label (GTK_MENU_ITEM (p->item), action_label);
+                g_free (action_label);
+
                 gtk_widget_set_visible (p->item, TRUE);
                 actions_visible = TRUE;
             } else {
@@ -803,7 +820,7 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 
         gtk_widget_set_visible (view->details->popup_action_separator, actions_visible);
 
-        nemo_file_list_free (tmp);
+        g_list_free (tmp); // Don't free the file, just the list, the file is owned by the model.
 
 		gtk_menu_popup (GTK_MENU (view->details->popup),
 				NULL, NULL, NULL, NULL,
@@ -1049,6 +1066,14 @@ fm_tree_view_get_containing_window (FMTreeView *view)
 	return GTK_WINDOW (window);
 }
 
+static void
+fm_tree_view_pin_unpin_cb (GtkWidget  *menu_item,
+                           FMTreeView *view)
+{
+    nemo_file_set_pinning (view->details->popup_file,
+                           !nemo_file_get_pinning (view->details->popup_file));
+}
+
 static void
 fm_tree_view_trash_cb (GtkWidget *menu_item,
 		       FMTreeView *view)
@@ -1169,13 +1194,6 @@ popup_menu_deactivated (GtkMenuShell *menu_shell, gpointer data)
 	}
 }
 
-static void
-action_payload_free (gpointer data)
-{
-    ActionPayload *p = (ActionPayload *) data;
-    gtk_widget_destroy (GTK_WIDGET (p->item));
-}
-
 static void
 action_activated_callback (GtkMenuItem *item, ActionPayload *payload)
 {
@@ -1199,7 +1217,7 @@ static void
 add_action_popup_items (FMTreeView *view)
 {
     if (view->details->action_items != NULL)
-        g_list_free_full (view->details->action_items, action_payload_free);
+        g_list_free_full (view->details->action_items, g_free);
 
     view->details->action_items = NULL;
 
@@ -1226,6 +1244,43 @@ add_action_popup_items (FMTreeView *view)
     }
 }
 
+/* Callback used when the file list's popup menu is detached */
+static void
+popup_menu_detach_cb (GtkWidget *attach_widget,
+                      GtkMenu   *menu)
+{
+    FMTreeView *view;
+
+    view = FM_TREE_VIEW (attach_widget);
+    g_assert (FM_IS_TREE_VIEW (view));
+
+    view->details->popup = NULL;
+    view->details->popup_open = NULL;
+    view->details->popup_open_in_new_window = NULL;
+    view->details->popup_open_in_new_tab = NULL;
+    view->details->popup_create_folder = NULL;
+    view->details->popup_cut = NULL;
+    view->details->popup_copy = NULL;
+    view->details->popup_paste = NULL;
+    view->details->popup_rename = NULL;
+    view->details->popup_pin = NULL;
+    view->details->popup_trash = NULL;
+    view->details->popup_delete = NULL;
+    view->details->popup_properties = NULL;
+    view->details->popup_unmount_separator = NULL;
+    view->details->popup_unmount = NULL;
+    view->details->popup_eject = NULL;
+    view->details->popup_action_separator = NULL;
+}
+
+static void
+actions_changed_callback (FMTreeView *view)
+{
+    if (view->details->popup) {
+        gtk_menu_detach (GTK_MENU (view->details->popup));
+    }
+}
+
 static void
 create_popup_menu (FMTreeView *view)
 {
@@ -1238,11 +1293,14 @@ create_popup_menu (FMTreeView *view)
 	
 	popup = gtk_menu_new ();
 
+    gtk_menu_attach_to_widget (GTK_MENU (popup),
+                               GTK_WIDGET (view),
+                               popup_menu_detach_cb);
+
 	g_signal_connect (popup, "deactivate",
 			  G_CALLBACK (popup_menu_deactivated),
 			  view);
-	
-	
+
 	/* add the "open" menu item */
 	menu_image = gtk_image_new_from_icon_name ("folder-open-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_show (menu_image);
@@ -1335,7 +1393,34 @@ create_popup_menu (FMTreeView *view)
 	view->details->popup_paste = menu_item;
 	
 	eel_gtk_menu_append_separator (GTK_MENU (popup));
-	
+
+    menu_image = gtk_image_new_from_icon_name ("xapp-pin-symbolic", GTK_ICON_SIZE_MENU);
+    gtk_widget_show (menu_image);
+    menu_item = gtk_image_menu_item_new_with_mnemonic (_("P_in"));
+    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
+                                   menu_image);
+    g_signal_connect (menu_item, "activate",
+                      G_CALLBACK (fm_tree_view_pin_unpin_cb),
+                      view);
+    gtk_widget_show (menu_item);
+    gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
+    view->details->popup_pin = menu_item;
+
+    menu_image = gtk_image_new_from_icon_name ("xapp-unpin-symbolic", GTK_ICON_SIZE_MENU);
+    gtk_widget_show (menu_image);
+    menu_item = gtk_image_menu_item_new_with_mnemonic (_("Unp_in"));
+    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
+                                   menu_image);
+    g_signal_connect (menu_item, "activate",
+                      G_CALLBACK (fm_tree_view_pin_unpin_cb),
+                      view);
+    gtk_widget_show (menu_item);
+    gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
+    view->details->popup_unpin = menu_item;
+
+
+    eel_gtk_menu_append_separator (GTK_MENU (popup));
+
 	/* add the "move to trash" menu item */
 	menu_image = gtk_image_new_from_icon_name (NEMO_ICON_SYMBOLIC_TRASH_FULL,
 						   GTK_ICON_SIZE_MENU);
@@ -1482,6 +1567,11 @@ create_tree (FMTreeView *view)
 	fm_tree_model_add_root_uri (view->details->child_model, home_uri, _("Home"), icon, NULL);
 	g_object_unref (icon);
 	g_free (home_uri);
+
+    icon = g_themed_icon_new ("favorites");
+    fm_tree_model_add_root_uri (view->details->child_model, "favorites:///", _("Favorites"), icon, NULL);
+    g_object_unref (icon);
+
 	icon = g_themed_icon_new (NEMO_ICON_FILESYSTEM);
 	fm_tree_model_add_root_uri (view->details->child_model, "file:///", _("File System"), icon, NULL);
 	g_object_unref (icon);
@@ -1526,6 +1616,11 @@ create_tree (FMTreeView *view)
 
     view->details->action_manager = nemo_action_manager_new ();
 
+    view->details->action_manager_changed_id = g_signal_connect_swapped (view->details->action_manager,
+                                                                         "changed",
+                                                                         G_CALLBACK (actions_changed_callback),
+                                                                         view);
+
     view->details->action_items = NULL;
 
 	/* Create column */
@@ -1545,9 +1640,10 @@ create_tree (FMTreeView *view)
 	cell = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, cell, TRUE);
 	gtk_tree_view_column_set_attributes (column, cell,
-					     "text", FM_TREE_MODEL_DISPLAY_NAME_COLUMN,
-					     "style", FM_TREE_MODEL_FONT_STYLE_COLUMN,
-					     NULL);
+                                         "text", FM_TREE_MODEL_DISPLAY_NAME_COLUMN,
+                                         "style", FM_TREE_MODEL_FONT_STYLE_COLUMN,
+                                         "weight", FM_TREE_MODEL_TEXT_WEIGHT_COLUMN,
+                                         NULL);
 
 	gtk_tree_view_append_column (view->details->tree_widget, column);
 
@@ -1624,6 +1720,27 @@ loading_uri_callback (NemoWindow *window,
 	schedule_select_and_show_location (view, location);
 }
 
+static void
+sort_directories_first_changed_callback (gpointer callback_data)
+{
+    FMTreeView *view;
+    gboolean preference_value;
+
+    view = FM_TREE_VIEW (callback_data);
+
+    preference_value = g_settings_get_boolean (nemo_preferences,
+                                               NEMO_PREFERENCES_SORT_DIRECTORIES_FIRST);
+
+    if (preference_value != view->details->sort_directories_first) {
+        view->details->sort_directories_first = preference_value;
+    }
+
+    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (view->details->sort_model));
+
+    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (view->details->sort_model),
+                         compare_rows, view, NULL);
+}
+
 static void
 fm_tree_view_init (FMTreeView *view)
 {
@@ -1649,6 +1766,13 @@ fm_tree_view_init (FMTreeView *view)
 				  "changed::" NEMO_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
 				  G_CALLBACK (filtering_changed_callback), view);
 
+    g_signal_connect_swapped (nemo_preferences,
+                  "changed::" NEMO_PREFERENCES_SORT_DIRECTORIES_FIRST,
+                  G_CALLBACK (sort_directories_first_changed_callback), view);
+
+    view->details->sort_directories_first = g_settings_get_boolean (nemo_preferences,
+                                                                    NEMO_PREFERENCES_SORT_DIRECTORIES_FIRST);
+
 	view->details->popup_file = NULL;
 
 	view->details->clipboard_handler_id =
@@ -1725,12 +1849,22 @@ fm_tree_view_dispose (GObject *object)
         view->details->hidden_files_changed_id = 0;
     }
 
+    if (view->details->action_manager_changed_id != 0) {
+        g_signal_handler_disconnect (view->details->action_manager,
+                                     view->details->action_manager_changed_id);
+        view->details->action_manager_changed_id = 0;
+    }
+
     g_clear_object (&view->details->action_manager);
 
 	g_signal_handlers_disconnect_by_func (nemo_tree_sidebar_preferences,
 					      G_CALLBACK(filtering_changed_callback),
 					      view);
 
+    g_signal_handlers_disconnect_by_func (nemo_tree_sidebar_preferences,
+                          G_CALLBACK(sort_directories_first_changed_callback),
+                          view);
+
 	view->details->window = NULL;
 
 	G_OBJECT_CLASS (parent_class)->dispose (object);
